{
Copyright  Frdric MONTIN, 2008

dekibulle@free.fr
http://dekibulle.free.fr

**fr**

Ce logiciel est un programme informatique servant  lire des fichiers
audio sur ordinateur.

Ce fichier est une partie de dkibulle.

Dkibulle est un logiciel libre; soumis  une double licence. 
Vous pouvez le redistribuer ainsi que le modifier ds lors que vous respectez 
les termes *de l'une des deux licences*
- *soit* la licence GNU Lesser General Public License comme publie par la 
  Free Software Foundation, dans sa version 2 ou ultrieure; 
- *soit* la licence Cecill-C comme publie par Cecill,
  dans sa version 1 ou ultrieure.

**en**

This software is a computer program whose purpose is to listen audio
files on a computer.

This file is a part of Dkibulle.

Dkibulle is a free software; subjected to a double license.
You can redistribute it and/or modify since you respect the terms of either 
(at least one of the both license) :
- the GNU Lesser General Public License as published by the Free Software Foundation; 
  either version 2.1 of the License, or any later version. 
- the CeCILL-C as published by CeCILL; either version 1 of the License, or any later version.
}

unit Unit_SongTagInfos;

interface

uses
  Dialogs, Classes, SysUtils, StrUtils, ID3v1, ID3v2, WMAFile, OGGVorbis, APETag, Bass;

type
  TSongInfo = class
    private
      MP3Tagv1 : TID3v1;
      MP3Tagv2 : TID3v2;
      WMATag   : TWMAfile;
      OGGTag   : TOggVorbis;
      APETag   : TAPETag;
    public
      FileName, Artist, Album, Title, Duree, Track, Genre, Annee, Comment : string;
      DureeBytes : Int64;
      DureeSecondes : Single;
      Tagged : Boolean;
      AlbPic : String;
      constructor Create();
      procedure   ReadFromFile(const sFile: String; const CheckLength: Boolean = True; const ReadAlbumPic: Boolean = False);
      procedure   ReadLengthOfFile(const sFile: String);
      procedure   ResetData;
  end;

implementation

constructor TSongInfo.Create();
begin
  inherited;
  ResetData;
end;

procedure TSongInfo.ReadFromFile(const sFile: String; const CheckLength: Boolean = True; const ReadAlbumPic: Boolean = False);
var
  Ext : string;
begin
  ResetData;
  FileName := sFile;
  Ext      := ExtractFileExt(sFile);
  if not(FileExists(sFile)) then
      Exit;

        { Dtermine la dure de la chanson si ncessaire }
    if CheckLength then
      ReadLengthOfFile(sFile);

          { Si le fichier est un mp3 ou mp2 on tente de regarder les TAG ID3 v2 }
    if AnsiContainsText('$.mp3$.mp2$', Ext) then begin
            { Lecture TAG ID3 v2 }
        MP3Tagv2 := TID3v2.Create;
        MP3Tagv2.ReadFromFile(sFile, ReadAlbumPic);
        if MP3Tagv2.Exists and (Trim(MP3Tagv2.Title) <> '') then begin
            Artist := MP3Tagv2.Artist;
            Album  := MP3Tagv2.Album;
            Title  := MP3Tagv2.Title;
            Track  := Format('%.2d', [MP3Tagv2.Track]);
            Genre  := MP3Tagv2.Genre;
            Annee  := MP3Tagv2.Year;
            Comment:= MP3Tagv2.Comment;
            AlbPic := MP3Tagv2.AlbPic;
            Tagged := True;
            MP3Tagv2.Free;
            Exit;
        end else
            MP3Tagv2.Free;
    end;

        { Si le fichier est un mpc, wv, ofr, ofs, ape ou mac on tente de regarder les TAG APE }
    if AnsiContainsText('$.mpc$.wv$.ofr$.ofs$.ape$.mac$', Ext) then begin
            { Lecture TAG APE }
        APETag := TAPETag.Create;
        APETag.ReadFromFile(sFile);
        if APETag.Exists and (Trim(APETag.SeekField('Title')) <> '')  then begin
            Artist := APETag.SeekField('Artist');
            Album  := APETag.SeekField('Album');
            Title  := APETag.SeekField('Title');
            Track  := APETag.SeekField('Track');
            Genre  := APETag.SeekField('Genre');
            Annee  := APETag.SeekField('Year');
            Comment:= APETag.SeekField('Comment');
            Tagged := True;
            APETag.Free;
            Exit;
        end else
            APETag.Free;
    end;

        { Si le fichier est un mp3, mp2 ou mpc on tente de regarder les TAG ID3 v1.1 }
    if AnsiContainsText('$.mp3$.mp2$.mpc$', Ext) then begin
            { Lecture TAG ID3 v1.1 }
        MP3Tagv1 := TID3v1.Create;
        MP3Tagv1.ReadFromFile(sFile);
        if MP3Tagv1.Exists and (Trim(MP3Tagv1.Title) <> '')  then begin
            Artist := MP3Tagv1.Artist;
            Album  := MP3Tagv1.Album;
            Title  := MP3Tagv1.Title;
            Track  := Format('%.2d', [MP3Tagv1.Track]);
            Genre  := MP3Tagv1.Genre;
            Annee  := MP3Tagv1.Year;
            Comment:= MP3Tagv1.Comment;
            Tagged := True;
            MP3Tagv1.Free;
            Exit;
        end else
            MP3Tagv1.Free;
    end;

        { Si le fichier est un wma on tente de regarder les TAG WMA }
    if AnsiContainsText('$.wma$', Ext) then begin
        WMATag := TWMAfile.Create;
        WMATag.ReadFromFile(sFile);
        if WMATag.Valid and (Trim(WMATag.Title) <> '') then begin
            Artist := WMATag.Artist;
            Album  := WMATag.Album;
            Title  := WMATag.Title;
            Track  := Format('%.2d', [WMATag.Track]);
            Genre  := WMATag.Genre;
            Annee  := WMATag.Year;
            Comment:= WMATag.Comment;
            Tagged := True;
            WMATag.Free;
            Exit;
        end else
            WMATag.Free;
    end;

        { Si le fichier est un ogg on tente de regarder les TAG OGG }
    if AnsiContainsText('$.ogg$', Ext) then begin
        OGGTag := TOggVorbis.Create;
        OGGTag.ReadFromFile(sFile);
        if OGGTag.Valid and (Trim(OGGTag.Title) <> '') then begin
            Artist := OGGTag.Artist;
            Album  := OGGTag.Album;
            Title  := OGGTag.Title;
            Track  := Format('%.2d', [OGGTag.Track]);
            Genre  := OGGTag.Genre;
            Annee  := OGGTag.Date;
            Comment:= OGGTag.Comment;
            Tagged := True;
            OGGTag.Free;
            Exit;
        end else
            OGGTag.Free;
    end;

end;


procedure TSongInfo.ReadLengthOfFile(const sFile: String);
const
  TimeAdj = 86400;
var
  Ch  : integer;
begin
  Ch := BASS_StreamCreateFile(FALSE, PChar(sFile), 0, 0, 0);
  if Ch = 0 then
    Ch := BASS_MusicLoad(FALSE, PChar(sFile), 0, 0, BASS_MUSIC_PRESCAN, 0);
  if Ch <> 0 then begin
    DureeBytes    := BASS_ChannelGetLength(Ch, BASS_POS_BYTE);
    DureeSecondes := BASS_ChannelBytes2Seconds(Ch, DureeBytes);
    if DureeSecondes / TimeAdj >= 1/24 then                      // Si la chanson dure plus d'une heure
      Duree := FormatDateTime('hh:nn:ss', DureeSecondes / TimeAdj)
    else
      Duree := FormatDateTime('nn:ss', DureeSecondes / TimeAdj);
  end;
  BASS_StreamFree(Ch);
end;


procedure TSongInfo.ResetData;
begin
  FileName := '';
  Artist   := '';
  Album    := '';
  Title    := '';
  Duree    := '';
  DureeBytes := -1;
  DureeSecondes := -1;
  Track    := '';
  Genre    := '';
  Annee    := '';
  Comment  := '';
  AlbPic   := '';
  Tagged   := False;
end;


end.

